from pathlib import Path
from time import sleep


# from venv import create

import click

from bagels.locations import config_file, database_file, set_custom_root
from bagels.versioning import get_current_version, get_pypi_version, needs_update


@click.group(invoke_without_command=True)
@click.option(
    "--at",
    type=click.Path(exists=True, file_okay=True, dir_okay=True, path_type=Path),
    help="Specify the path.",
)
@click.pass_context
def cli(ctx, at: click.Path | None):
    """Bagels CLI."""
    if at:
        set_custom_root(at)
    if ctx.invoked_subcommand is None:

        from bagels.config import load_config

        load_config()

        from bagels.config import CONFIG

        if CONFIG.state.check_for_updates:
            if needs_update():
                new = get_pypi_version()
                cur = get_current_version()
                click.echo(
                    click.style(
                        f"New version available ({cur} -> {new})! Update with:",
                        fg="yellow",
                    )
                )
                click.echo(click.style("```uv tool upgrade bagels```", fg="cyan"))
                click.echo(
                    click.style(
                        "You can disable this check in-app using the command palette.",
                        fg="bright_black",
                    )
                )
                sleep(2)

        from bagels.models.database.app import init_db

        init_db()

        from bagels.app import App

        app = App()
        app.run()


@cli.command()
@click.argument("thing_to_locate", type=click.Choice(["config", "database"]))
def locate(thing_to_locate: str) -> None:
    if thing_to_locate == "config":
        print("Config file:")
        print(config_file())
    elif thing_to_locate == "database":
        print("Database file:")
        print(database_file())


if __name__ == "__main__":
    cli()



from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Container
from textual.widgets import Input, Label, Static, Switch

from bagels.components.autocomplete import AutoComplete, Dropdown, DropdownItem
from bagels.forms.form import Form, FormField
from bagels.utils.format import parse_formula_expression

_RESTRICT_TYPES = {
    "any": None,
    "integer": r"^-?\d+$",
    "number": r"^-?\d*\.?\d*(?:[+\-*\/]-?\d*\.?\d+)*[+\-*\/\.]?$",
}


class Fields(Static):
    """Container for multiple form fields"""

    def __init__(self, fields: Form):  # should rename, but whatever
        super().__init__()
        self.form = fields

    def compose(self) -> ComposeResult:
        for field in self.form.fields:
            yield Field(field)


class Field(Static):
    """Individual form field that can be text, number, boolean, or autocomplete"""

    BINDINGS = [
        Binding("tab", "focus_next", "Focus next", False),
        Binding("shift+tab", "focus_prev", "Focus previous", False),
    ]

    def __init__(self, field: FormField):
        super().__init__()
        self.field = field

        # Create base input widget
        self.input = Input(placeholder=field.placeholder or "", id=f"field-{field.key}")

        # Configure input based on field type
        match self.field.type:
            case "hidden":
                pass
            case "integer" | "number":
                self.input.restrict = _RESTRICT_TYPES.get(self.field.type, None)
                self.input.value = field.default_value or ""

            case "autocomplete":
                self.input.heldValue = field.default_value or ""
                self.input.value = field.default_value_text or field.default_value or ""

            case type_ if type_ != "boolean":
                self.input.value = field.default_value or ""

    def on_auto_complete_selected(self, event: AutoComplete.Selected) -> None:
        """Handle autocomplete selection"""
        # self.screen.focus_next()

        # Find matching option and set held value
        for item in self.field.options.items:
            item_text = str(item.text or item.value)
            selected_dropdown_text = str(event.item.main)
            if item_text == selected_dropdown_text:
                self.input.heldValue = item.value
                break

    def on_input_changed(self, event: Input.Changed):
        if self.field.type == "number":
            num_val = parse_formula_expression(event.value)
            self.query_one(".label").update(f"{self.field.title} - {num_val}")

    def action_focus_next(self):
        self.screen.focus_next()

    def action_focus_prev(self):
        self.screen.focus_previous()

    def compose(self) -> ComposeResult:
        if self.field.type == "hidden":
            # Hidden fields just need a static widget to hold the value
            self.input = Static(id=f"field-{self.field.key}", classes="hidden-field")
            self.input.value = self.input.heldValue = self.field.default_value
            yield self.input
            return

        # Visible fields get a container with label and input
        with Container(classes="field-row", id=f"row-field-{self.field.key}"):
            yield Label(f"{self.field.title}", classes="label")

            if self.field.type == "autocomplete":
                # Build dropdown items list
                dropdown_items = [
                    DropdownItem(
                        item.text or item.value,
                        item.prefix or "",
                        item.postfix or "",
                    )
                    for item in self.field.options.items
                ]
                dropdown = Dropdown(
                    items=dropdown_items,
                    show_on_focus=True,
                    id=f"dropdown-{self.field.key}",
                    create_option=self.field.create_action,
                    show_when_empty=self.field.autocomplete_selector,
                )

                yield AutoComplete(
                    self.input,
                    dropdown,
                    classes="field-autocomplete",
                    create_action=self.field.create_action,
                    backspace_clears=self.field.autocomplete_selector,
                )

            elif self.field.type == "boolean":
                with Container(classes="switch-group"):
                    yield Label(str(self.field.labels[0]), classes="left")
                    yield Switch(
                        id=f"field-{self.field.key}",
                        value=self.field.default_value or False,
                    )
                    yield Label(str(self.field.labels[1]), classes="right")

            else:
                yield self.input



import requests
from packaging import version
from importlib.metadata import metadata


def get_pypi_version():
    """Fetch the latest version from PyPI."""
    try:
        response = requests.get("https://pypi.org/pypi/bagels/json")
        return response.json()["info"]["version"]
    except Exception:
        return None


def get_current_version():
    return metadata("bagels")["Version"]


def needs_update():
    """Check if the current version needs an update."""
    pypi_version = get_pypi_version()
    current_version = get_current_version()
    if not pypi_version:
        return False

    return version.parse(pypi_version) > version.parse(current_version)
